%% Multidimensional Harmonic Retrieval
%   The goal of this demo is to illustrate the use of the basic cpd command
%   in array processing applications. We consider binary phase-shift keying
%   (BPSK) signals impinging on a 10x10 uniform rectangular array or URA.
%   The goal is to separate these signals and identify the directions of
%   arrival. The harmonic structure due to the array geometry is not
%   exploited in this demo; this can be done in the structured data fusion
%   framework as illustrated in other demos such as 'Using advanced
%   Tensorlab features for ICA'. This demo also illustrates the use of cpd
%   in the case where values are missing due to sensor malfunctioning
%
%    A more detailed discussion can be found <a href="matlab:
%    web('http://www.tensorlab.net/demos/harmonic-retrieval.html', '-browser')">online</a>.
%
%    See also cpd, cpderr

% Authors: Otto Debals         (Otto.Debals@esat.kuleuven.be)
%          Lieven De Lathauwer (Lieven.DeLathauwer@kuleuven-kulak.be)
%
% Version History:
% - 2015/07/12   OD      Initial version

clear variables; close all; clc
rng('default'); rng(1)

fprintf(['The goal of this demo is to illustrate the use of the basic ''cpd''\n' ...
    'command in array processing applications. We consider binary phase-shift\n' ...
    'keying (BPSK) signals impinging on a 10x10 uniform rectangular array\n' ...
    'or URA. The goal is to separate these signals and identify the directions\n' ...
    'of arrival. The harmonic structure due to the array geometry is not\n' ...
    'exploited in this demo; this can be done in the structured data fusion\n' ...
    'framework as illustrated in other demos such as ''Using advanced Tensorlab\n' ...
    'features for ICA''. This demo also illustrates the use of ''cpd'' in\n' ...
    'the case where values are missing due to sensor malfunctioning\n']);
fprintf('An accompanying text can be found %s.\n\n',gendemolink('harmonic-retrieval','online'));
fprintf(['Go to <a href="matlab: web(''http://www.tensorlab.net'', ''-browser'')">www.tensorlab.net</a> for Tensorlab\''s documentation and further demos.\n' ...
    '--------------------------------------------------------------------------\n\n']);

%% Data
% Generation
N = 15; R = 3; I = 10;
azimuth_angles = [10 30 70];    % azimuth angles in degrees
elevation_angles = [20 30 40];  % elevation angles in degrees
lambda = 0.25;                  % wavelength of signal carriers
dx = 0.1;                       % sensor distance
for r = 1:R
  zar = exp(2*pi/lambda*sin(azimuth_angles(r)*pi/180)*dx*1i);
  for i = 1:I, A(i,r) = zar^(i-1); end
  zer = exp(2*pi/lambda*sin(elevation_angles(r)*pi/180)*dx*1i);
  for i = 1:I, E(i,r) = zer^(i-1); end
end

S = round(rand(N,R))*2-1;
attenuation_vector = [1 .5 .25];
for r = 1:R
  S(:,r) = attenuation_vector(r)*S(:,r);
end

% Visualization of the sources
figure(); axis equal;
t = 1:N;
for i = 1:R
    subplot(R,1,i);
    plot(t,S(:,i),'*-')
    xlabel('t');
    ylabel(sprintf('Source %d',i))
    axis([0 N -1.3 1.3])
end

% Tensor generation
T = cpdgen({A,E,S});
SNR = 0;
Tnoisy = noisy(T,SNR);

% Visualisation of the tensor
figure(); slice3(T)
title('Slices of the observed tensor')

% Visualisation of two observed signals
figure();
subplot(2,1,1);
plot(real(squeeze(T(4,5,:))),'o-'); hold all;
plot(real(squeeze(Tnoisy(4,5,:))),'*-')
title('Real part observed signal (4,5)');
legend('Exact signal','Noisy signal','Location','Best');
xlabel('t')
subplot(2,1,2);
plot(real(squeeze(T(5,6,:))),'o-'); hold all
plot(real(squeeze(Tnoisy(5,6,:))),'*-')
title('Real part observed signal (5,6)')
xlabel('t')

%% Signal separation and direction-of-arrival estimation
fprintf('Estimating the factor matrices\n');
% CPD
Uest = cpd(Tnoisy,R);
Aest = Uest{1};
Eest = Uest{2};
Sest = Uest{3};

% Calculating the error on the factor matrices and fixing the
% indeterminacies for visualization purposes
[err,P,D] = cpderr({A,E,S},Uest);
Aestfix = Aest*P*D{1};
fprintf('\t- Error on azimuth factor matrix = %e\n',err(1))
Eestfix = Eest*P*D{2};
fprintf('\t- Error on elevation factor matrix = %e\n',err(2))
Sestfix = Sest*P*D{3};
fprintf('\t- Error on source matrix = %e\n',err(3))

% Visualizing the permuted and scaled recovered sources
figure(); axis equal;
for i = 1:R
    subplot(R,1,i); plot(t,S(:,i),'o-');
    hold on;
    plot(t,real(Sestfix(:,i)),'*-');   
    if i==1, legend('Exact','Estimated','Location','Best'); end
    ylabel(sprintf('Source %d',i));
    xlabel('n');
end

% Estimation of direction-of-arrival angles
fprintf('\nEstimation of direction-of-arrival angles\n');
azimuth_angles_est = zeros(size(azimuth_angles));
elevation_angles_est = zeros(size(elevation_angles));
for i = 1:R
    azimuth_z_est = Aestfix(1:end-1,i)\Aestfix(2:end,i);
    azimuth_angles_est(i) = asin(angle(azimuth_z_est)*lambda/(2*pi*dx))*(180/pi);
    elevation_z_est = Eestfix(1:end-1,i)\Eestfix(2:end,i);
    elevation_angles_est(i) = asin(angle(elevation_z_est)*lambda/(2*pi*dx))*(180/pi);
end

azimuth_err = abs(sort(azimuth_angles)-sort(azimuth_angles_est))./abs(sort(azimuth_angles));
fprintf('\t- Error on azimuth angles = %e, %e and %e\n',azimuth_err.')
elevation_err = abs(sort(elevation_angles)-sort(elevation_angles_est))./abs(sort(elevation_angles));
fprintf('\t- Error on elevation angles = %e, %e and %e\n',elevation_err.')

%% Missing values due to broken sensors
fprintf('\nMissing values due to broken sensors\n');
% Data
Tnoisy(2,3,:) = NaN;
Tnoisy(5,1,1:floor(end/2)) = NaN;
Tnoisy(9,7,ceil(end/2):end) = NaN;

figure(); 
slice3(Tnoisy)

% CPD
Uest = cpd(Tnoisy,R);
[err,P,D] = cpderr({A,E,S},Uest);
fprintf('\t- Error on azimuth factor matrix = %e\n',err(1))
fprintf('\t- Error on elevation factor matrix = %e\n',err(2))
fprintf('\t- Error on source matrix = %e\n',err(3))
